-- ----------------------------------------------------------------------------
-- Product: SAS Drug Development
-- Database: Oracle
-- Description: DDL for upgrading SDD from version 4.1.1 to 4.2
--    This corresponds to the platform upgrade from 1.0.1 to 1.1
--    This script assumes access to both the docstore schema AND global schema.
--    These schemas are hardcoded below as xdstor1 and xgs1.
--    If the default schema names are NOT used then these schema prefixes need
--       to be changed below before executing.
-- Upgrade: 1.0.1 to 1.1
-- Copyright: 2011, SAS Institute Inc., Cary, NC
-- ----------------------------------------------------------------------------
-- $Id: //depot/platform/1.1.1.A/src/sql/oracle/upgrade/sas_xds_upgrade_to_1.1.sql#1 $
-- $DateTime: 2013/06/13 13:50:55 $
-- $Change: 34880 $
-- $Author: jeff.piper $
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- modify message table column sizes
-- ----------------------------------------------------------------------------
alter table xdstore1.sas_messages modify (sender varchar2(36));
alter table xdstore1.sas_messages modify (recipient varchar2(36));

  
-- ----------------------------------------------------------------------------
-- modify group table column names
-- ----------------------------------------------------------------------------
alter table xdstore1.sas_group rename column version to revision;
alter table xdstore1.sas_group rename column object_identifier to object_id;


-- ----------------------------------------------------------------------------
-- modify group member table column names
-- ----------------------------------------------------------------------------
alter table xdstore1.sas_group_member rename column principal_identifier to principal_id;


-- ----------------------------------------------------------------------------
-- modify task history table column sizes
-- ----------------------------------------------------------------------------
alter table xdstore1.sas_task_history modify (assignee varchar2(36));
alter table xdstore1.sas_task_history modify (assignee_type varchar2(128));


-- ----------------------------------------------------------------------------
-- add runAs columns to job submissions table 
-- ----------------------------------------------------------------------------
alter table xdstore1.sas_job_submissions
    add (runAsUser varchar2(32), runAsOwner number(1) default 0, jobRunContext varchar2(50));
   

-- ----------------------------------------------------------------------------
-- update runAsOwner column of job submissions table 
-- ----------------------------------------------------------------------------
update xdstore1.sas_job_submissions s set s.runAsOwner = 1 where s.jobIdentifier is not null;
update xdstore1.sas_job_submissions s set s.runAsOwner = 0 where s.jobIdentifier is null;


-- ----------------------------------------------------------------------------
-- update runAsUser column of job submissions table 
-- ----------------------------------------------------------------------------
update xdstore1.sas_job_submissions s set s.runAsUser = 
     ( select jobOwner from xdstore1.sas_job_submissions j where j.id = s.id) where s.jobIdentifier is not null;
     
update xdstore1.sas_job_submissions s set s.runAsUser = 
     ( select submittedBy from xdstore1.sas_job_submissions j where j.id = s.id) where s.jobIdentifier is null;

     
-- ----------------------------------------------------------------------------
-- initialize job run context column of  job submissions table 
-- ----------------------------------------------------------------------------
update xdstore1.sas_job_submissions s set s.jobruncontext = '';


-- ----------------------------------------------------------------------------
-- create the new membership tables
-- ----------------------------------------------------------------------------
create table xdstore1.sas_membership (
  object_type varchar2(128) not null,
  object_id varchar2(36) not null,
  revision number(10) default -1 not null,
  parent_id varchar2(36),
  constraint pk_sas_membership primary key (object_id) using index tablespace users,
  constraint fk_sas_membership_parent foreign key (parent_id) references xdstore1.sas_membership(object_id) on delete cascade
) tablespace users;

create table xdstore1.sas_membership_member (
  membership_id varchar2(36) not null,
  principal_type varchar2(128) not null,
  principal_id varchar2(36) not null,
  assigned number(1) default 0 not null,
  constraint pk_sas_membership_member primary key (membership_id, principal_type, principal_id) using index tablespace users,
  constraint fk_sas_membership_member foreign key (membership_id) references xdstore1.sas_membership(object_id) on delete cascade
) tablespace users;


-- ----------------------------------------------------------------------------
-- move the membersip data from the relationship tables to the new membership tables
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- migrate all the distinct sas_relationship data to the sas_membership table
--   - revision will default to 1
--   - parent_id will default to null (set below in muti-step process)
-- ----------------------------------------------------------------------------
insert into xdstore1.sas_membership (object_type, object_id, revision, parent_id)
  (select distinct left_object_type, left_object_identifier, '1', null from xdstore1.sas_relationships where rtype = 'PRINCIPAL_ASSOCIATION');

-- ----------------------------------------------------------------------------
-- update the parent_id values based on the xythos property data
--   - excludes tenant and organization - these are special and handled seperately
-- ----------------------------------------------------------------------------
update xdstore1.sas_membership set parent_id =
  (select p_parent_id.string_value

      from xdstore1.xyf_urls u, 
        xdstore1.xyf_files f, 
        xdstore1.xyf_properties p_id, 
        xdstore1.xyf_properties p_type,
        xdstore1.xyf_properties p_parent_id

      where u.file_id = f.file_id

      -- identifier attribute
      and f.latest_version=p_id.version_number
      and p_id.property_owner_id=u.file_id
      and p_id.property_definition_id=
                           (select id
                            from xgs1.xy_property_definitions
                            where property_name = 'identifier')
      
      -- type attribute
      and f.latest_version=p_type.version_number
      and p_type.property_owner_id=u.file_id
      and p_type.property_definition_id=
                           (select id
                            from xgs1.xy_property_definitions
                            where property_name = 'type')
                            
      -- parent identifier attribute
      -- and f.latest_version=p_parent_id.version_number
      and p_parent_id.property_owner_id=u.parent_id
      and p_parent_id.property_definition_id=
                           (select id
                            from xgs1.xy_property_definitions
                            where property_name = 'identifier')
                            
      and p_type.string_value = object_type
      and p_id.string_value = object_id)

where object_type not in ('sas:tenant', 'sas:organization');
  
-- ----------------------------------------------------------------------------
-- update the organization parent id
-- ----------------------------------------------------------------------------
update xdstore1.sas_membership set parent_id = '1' where object_type = 'sas:organization';

-- ----------------------------------------------------------------------------
-- update the tenant parent id - null
-- ----------------------------------------------------------------------------
update xdstore1.sas_membership set parent_id = null where object_type = 'sas:tenant';  

-- ----------------------------------------------------------------------------
-- migrate the sas_relationship data to the sas_membership_member table
--   - assigned will default to 0 (set below in muti-step process
-- ----------------------------------------------------------------------------
insert into xdstore1.sas_membership_member (membership_id, principal_type, principal_id, assigned)
  (select left_object_identifier, right_object_type, right_object_identifier, 0 from xdstore1.sas_relationships where rtype = 'PRINCIPAL_ASSOCIATION');

-- ----------------------------------------------------------------------------
-- if the member is a user, and the membership assignment is NOT at the tenant, 
-- then set assigned to 1 
-- ----------------------------------------------------------------------------
update xdstore1.sas_membership_member set assigned = 1 where principal_type = 'sas:user' and membership_id <> '1';

-- ----------------------------------------------------------------------------
-- if the member is a group, and the membership assignment is NOT where group 
-- is defined, then set assigned to 1
-- ----------------------------------------------------------------------------
update xdstore1.sas_membership_member set assigned = 1 where principal_type = 'sas:group' and membership_id <> 
  (select g.object_id from xdstore1.sas_group g where g.id = principal_id);

-- ----------------------------------------------------------------------------
-- rename the unused relationship table (it will be removed in later release)
-- ----------------------------------------------------------------------------
-- alter table xdstore1.sas_relationships rename to xdstore1.sas_relationships_DEPRECATED;
  
 
